/**************************************************************************/
/*                                                                        */
/*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       */
/*  Kim Nguyen, Laurent Sartre                                            */
/*                                                                        */
/*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      */
/*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 */
/*                                                                        */
/*  https://www.informatique-mpi.fr/                                      */
/*                                                                        */
/**************************************************************************/

// Arbres préfixes

#include "prelude.h"

#include "trie.h"

struct Trie {
  bool present; // il y a une valeur pour cette entrée
  int value;    // elle est alors dans le champ value
  struct Trie *branches[256];
};

trie *trie_create(void) {
  trie *t = malloc(sizeof(struct Trie));
  t->present = false;
  for (int i = 0; i < 256; i++) t->branches[i] = NULL;
  return t;
}

void trie_put(trie *t, char *s, int v) {
  assert(t != NULL);
  for (; *s != 0; s++) {
    unsigned char c = *s;
    if (t->branches[c] == NULL) t->branches[c] = trie_create();
    t = t->branches[c];
  }
  t->present = true;
  t->value = v;
}

// renvoie le noeud correspondant à s, s'il existe, et NULL sinon
trie *trie_find(trie *t, char *s) {
  assert(t != NULL);
  for (; *s != 0; s++) {
    unsigned char c = *s;
    t = t->branches[c];
    if (t == NULL) return NULL;
  }
  return t;
}

bool trie_contains(trie *t, char *s) {
  trie *n = trie_find(t, s);
  return n != NULL && n->present;
}

int trie_get(trie *t, char *s) {
  trie *n = trie_find(t, s);
  assert(n != NULL);
  return n->value;
}

int trie_prefix(trie *t, char *s) {
  assert(t != NULL);
  int p = 0, i = 0;
  for (; *s != 0; s++) {
    unsigned char c = *s;
    t = t->branches[c];
    if (t == NULL) return p;
    i++;
    if (t->present) p = i;
  }
  return p;
}

void trie_delete(trie *t) {
  if (t == NULL) return;
  for (int i = 0; i < 256; i++)
    trie_delete(t->branches[i]);
  free(t);
}

